==> thx_address-0.0.6.tar.gz: thx_address-0.0.6/thx/address/models.py <==
# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _

from .managers import NearbyManager
from .utils import get_distance, get_google_geocoding
from thx.l10n.models import Country, AdministrativeArea
from thx.core.models import UserModel, DateModel, StatusModel, OrderedModel, CleanModel


class AddressScope(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    """
    Store a single Address Scope ('Invoice Address', 'Delivery Address', etc.)
    """
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_("name"),
    )
    slug = models.SlugField(
        max_length=255,
        unique=True,
        verbose_name=_("slug"),
    )
    is_invoice_address = models.BooleanField(
        default=False,
        verbose_name=_('invoice address'),
        help_text=_("if this address scope is invoice")
    )
    is_shipping_address = models.BooleanField(
        default=False,
        verbose_name=_('shipping address'),
        help_text=_("if this address scope is shipping")
    )

    class Meta:
        verbose_name = _('address scope')
        verbose_name_plural = _('address scopes')
        ordering = ('ordering', 'name')
        permissions = (
            ("list_addressscope", "Can list address scope"),
            ("detail_addressscope", "Can detail address scope"),
            ("enable_addressscope", "Can enable address scope"),
            ("disable_addressscope", "Can disable address scope"),
        )

    def __str__(self):
        return self.name


class Address(models.Model):
    """
    Abstract Address model
    """
    address_scopes = models.ManyToManyField(
        AddressScope,
        db_index=True,
        verbose_name=_("address scopes"),
    )
    address_type = models.IntegerField(
        default=7,
        choices=settings.ADDRESS_ADDRESS_TYPE_CHOICES,
        db_index=True,
        verbose_name=_("address type"),
        blank=True, null=True,
    )
    name = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    number = models.CharField(
        max_length=20,
        blank=True,
        verbose_name=_("number"),
        db_column="a_number"
    )
    crossed = models.CharField(
        max_length=5,
        blank=True,
        verbose_name=_("crossed"),
    )
    scale = models.CharField(
        max_length=5,
        blank=True,
        verbose_name=_("scale"),
    )
    floor = models.CharField(
        max_length=3,
        blank=True,
        verbose_name=_("floor"),
    )
    zip_code = models.CharField(
        max_length=20,
        db_index=True,
        verbose_name=_("ZIP code"),
        blank=True,
    )
    locality = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("locality"),
        blank=True,
    )
    province = models.CharField(
        max_length=3,
        db_index=True,
        verbose_name=_("province"),
        blank=True,
    )
    country = models.ForeignKey(
        Country,
        on_delete=models.PROTECT,
        verbose_name=_("country"),
    )
    post_office = models.CharField(
        max_length=10,
        blank=True,
        verbose_name=_("post office"),
    )
    administrative_area = models.ForeignKey(
        AdministrativeArea,
        blank=True, null=True,
        on_delete=models.PROTECT,
        verbose_name=_("administrative area"),
    )

    class Meta:
        abstract = True
        verbose_name = _('address')
        verbose_name_plural = _('addresses')
        ordering = ('ordering',)
        permissions = (
            ("list_address", "Can list address"),
            ("detail_address", "Can detail address"),
            ("enable_address", "Can enable address"),
            ("disable_address", "Can disable address"),
        )

    def __str__(self):
        return "{address_type} {name} {number} - {locality} ({province}) - {adm_area} - {country}".format(
            address_type=self.get_address_type_display(),
            name=self.name,
            number=self.number or '',
            locality=self.locality,
            province=self.province,
            adm_area=self.administrative_area,
            country=self.country
        )

    def clean(self):
        self.check_administrative_area()
        super().clean()

    def check_administrative_area(self):
        if self.administrative_area:

            if self.administrative_area.country != self.country:
                raise ValidationError({
                    'administrative_area': _('administrative area country does not match with address country')
                })

    def get_scopes(self):
        scopes = self.address_scopes.all()
        names = [scope.name for scope in scopes]
        return ", ".join(names)
    get_scopes.short_description = _("scopes")

    def get_gmaps_address(self):
        return "{address_type} {name}, {number}, {cap}, {locality}".format(
            address_type=self.get_address_type_display() or '',
            name=self.name,
            number=self.number,
            locality=self.locality,
            cap=self.zip_code or ''
        )
    gmaps_address = property(get_gmaps_address)


class AddressCoordinate(models.Model):
    """
    This abstract model is used when you want to add latitude and longitude
    coordinates to a concrete model. You must define in the inherited model an
    attribute called 'geocoding_fields' which is used to get and save
    coordinates via Google Maps API (v3)
    """
    lat = models.DecimalField(
        verbose_name=_("latitude"),
        blank=True, null=True,
        max_digits=20, decimal_places=17,
    )
    lng = models.DecimalField(
        verbose_name=_("longitude"),
        blank=True, null=True,
        max_digits=20, decimal_places=17,
    )
    geocoding_fields = []

    """
    NearbyManager gives access to the method .nearby(lat, lng, proximity),
    which returns all objects within the range in km (proximity)
    from the point(lat, lng)
    ie. AddressCoordinates.objects.nearby(lat, lng, proximity)
    """
    objects = NearbyManager()

    class Meta:
        abstract = True
        verbose_name = _('address coordinates')
        verbose_name_plural = _('address coordinates')
        ordering = ('ordering',)
        permissions = (
            ("list_addresscoordinate", "Can list address coordinate"),
            ("detail_addresscoordinate", "Can detail address coordinate"),
            ("enable_addresscoordinate", "Can enable address coordinate"),
            ("disable_addresscoordinate", "Can disable address coordinate"),
        )

    def __str__(self):
        return "lat: {}, lng: {}".format(self.lat, self.lng)

    @property
    def address_search_querystring(self):
        values = []
        for field in self.geocoding_fields:
            if isinstance(getattr(self, field), str) and getattr(self, field):
                values.append(getattr(self, field).strip())
        return '+'.join(values)

    def get_google_geocoding(self):
        return get_google_geocoding(self.address_search_querystring)

    def save(self, *args, **kwargs):
        if self.geocoding_fields and not self.pk:
            coordinates = self.get_google_geocoding()
            if coordinates['lat'] and coordinates['lng']:
                self.lat = coordinates['lat']
                self.lng = coordinates['lng']
        super(AddressCoordinate, self).save(*args, **kwargs)

    def google_map(self):
        context = dict()
        return render_to_string('admin/address/addresscoordinates/google_map.html', context)
    google_map.allow_tags = True
    google_map.short_description = _("map")

    def get_distance(self, latitude, longitude, radius=6371):
        """
        Uses the Haversine formula, resulting in an error of 0.5%
        By default, returns distance in km
        """
        return get_distance(longitude, latitude, self.lng, self.lat, radius)

==> thx_address-0.1.4.tar.gz: thx_address-0.1.4/thx/address/models.py <==
# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel
from thx.l10n.models import AdministrativeArea, Country

from .managers import NearbyManager
from .utils import get_distance, get_google_geocoding


class AddressScope(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    """
    Store a single Address Scope ('Invoice Address', 'Delivery Address', etc.)
    """
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_("name"),
    )
    slug = models.SlugField(
        max_length=255,
        unique=True,
        verbose_name=_("slug"),
    )
    is_invoice_address = models.BooleanField(
        default=False,
        verbose_name=_('invoice address'),
        help_text=_("if this address scope is invoice")
    )
    is_shipping_address = models.BooleanField(
        default=False,
        verbose_name=_('shipping address'),
        help_text=_("if this address scope is shipping")
    )

    class Meta:
        verbose_name = _('address scope')
        verbose_name_plural = _('address scopes')
        ordering = ('ordering', 'name')
        permissions = (
            ("list_addressscope", "Can list address scope"),
            ("detail_addressscope", "Can detail address scope"),
            ("enable_addressscope", "Can enable address scope"),
            ("disable_addressscope", "Can disable address scope"),
        )

    def __str__(self):
        return self.name


class Address(models.Model):
    """
    Abstract Address model
    """
    address_scopes = models.ManyToManyField(
        AddressScope,
        db_index=True,
        verbose_name=_("address scopes"),
    )
    address_type = models.IntegerField(
        default=7,
        choices=settings.ADDRESS_ADDRESS_TYPE_CHOICES,
        db_index=True,
        verbose_name=_("address type"),
        blank=True, null=True,
    )
    name = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    number = models.CharField(
        max_length=20,
        blank=True,
        verbose_name=_("number"),
        db_column="a_number"
    )
    crossed = models.CharField(
        max_length=5,
        blank=True,
        verbose_name=_("crossed"),
    )
    scale = models.CharField(
        max_length=5,
        blank=True,
        verbose_name=_("scale"),
    )
    floor = models.CharField(
        max_length=3,
        blank=True,
        verbose_name=_("floor"),
    )
    zip_code = models.CharField(
        max_length=20,
        db_index=True,
        verbose_name=_("ZIP code"),
        blank=True,
    )
    locality = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("locality"),
        blank=True,
    )
    province = models.CharField(
        max_length=3,
        db_index=True,
        verbose_name=_("province"),
        blank=True,
    )
    country = models.ForeignKey(
        Country,
        on_delete=models.PROTECT,
        verbose_name=_("country"),
    )
    post_office = models.CharField(
        max_length=10,
        blank=True,
        verbose_name=_("post office"),
    )
    administrative_area = models.ForeignKey(
        AdministrativeArea,
        blank=True, null=True,
        on_delete=models.PROTECT,
        verbose_name=_("administrative area"),
    )

    class Meta:
        abstract = True
        verbose_name = _('address')
        verbose_name_plural = _('addresses')
        ordering = ('ordering',)
        permissions = (
            ("list_address", "Can list address"),
            ("detail_address", "Can detail address"),
            ("enable_address", "Can enable address"),
            ("disable_address", "Can disable address"),
        )

    def __str__(self):
        return "{address_type} {name} {number} - {locality} ({province}) - {adm_area} - {country}".format(
            address_type=self.get_address_type_display(),
            name=self.name,
            number=self.number or '',
            locality=self.locality,
            province=self.province,
            adm_area=self.administrative_area,
            country=self.country
        )

    def clean(self):
        self.check_administrative_area()
        super().clean()

    def check_administrative_area(self):
        if self.administrative_area:

            if self.administrative_area.country != self.country:
                raise ValidationError({
                    'administrative_area': _('administrative area country does not match with address country')
                })

    def get_scopes(self):
        scopes = self.address_scopes.all()
        names = [scope.name for scope in scopes]
        return ", ".join(names)
    get_scopes.short_description = _("scopes")

    def get_gmaps_address(self):
        return "{address_type} {name}, {number}, {cap}, {locality}".format(
            address_type=self.get_address_type_display() or '',
            name=self.name,
            number=self.number,
            locality=self.locality,
            cap=self.zip_code or ''
        )
    gmaps_address = property(get_gmaps_address)


class AddressCoordinate(models.Model):
    """
    This abstract model is used when you want to add latitude and longitude
    coordinates to a concrete model. You must define in the inherited model an
    attribute called 'geocoding_fields' which is used to get and save
    coordinates via Google Maps API (v3)
    """
    lat = models.DecimalField(
        verbose_name=_("latitude"),
        blank=True, null=True,
        max_digits=20, decimal_places=17,
    )
    lng = models.DecimalField(
        verbose_name=_("longitude"),
        blank=True, null=True,
        max_digits=20, decimal_places=17,
    )
    geocoding_fields = []

    """
    NearbyManager gives access to the method .nearby(lat, lng, proximity),
    which returns all objects within the range in km (proximity)
    from the point(lat, lng)
    ie. AddressCoordinates.objects.nearby(lat, lng, proximity)
    """
    objects = NearbyManager()

    class Meta:
        abstract = True
        verbose_name = _('address coordinates')
        verbose_name_plural = _('address coordinates')
        ordering = ('ordering',)
        permissions = (
            ("list_addresscoordinate", "Can list address coordinate"),
            ("detail_addresscoordinate", "Can detail address coordinate"),
            ("enable_addresscoordinate", "Can enable address coordinate"),
            ("disable_addresscoordinate", "Can disable address coordinate"),
        )

    def __str__(self):
        return "lat: {}, lng: {}".format(self.lat, self.lng)

    @property
    def address_search_querystring(self):
        values = []
        for field in self.geocoding_fields:
            if isinstance(getattr(self, field), str) and getattr(self, field):
                values.append(getattr(self, field).strip())
        return '+'.join(values)

    def get_google_geocoding(self):
        return get_google_geocoding(self.address_search_querystring)

    def save(self, *args, **kwargs):
        if self.geocoding_fields and not self.pk:
            coordinates = self.get_google_geocoding()
            if coordinates['lat'] and coordinates['lng']:
                self.lat = coordinates['lat']
                self.lng = coordinates['lng']
        super(AddressCoordinate, self).save(*args, **kwargs)

    def google_map(self):
        context = dict()
        return render_to_string('admin/address/addresscoordinates/google_map.html', context)
    google_map.allow_tags = True
    google_map.short_description = _("map")

    def get_distance(self, latitude, longitude, radius=6371):
        """
        Uses the Haversine formula, resulting in an error of 0.5%
        By default, returns distance in km
        """
        return get_distance(longitude, latitude, self.lng, self.lat, radius)
